@@ -5,6 +5,7 @@ import psycopg2
 import dbt.exceptions
 from dbt.adapters.base import Credentials
 from dbt.adapters.sql import SQLConnectionManager
+from dbt.contracts.connection import AdapterResponse
 from dbt.logger import GLOBAL_LOGGER as logger
 
 from dbt.helper_types import Port
@@ -16,10 +17,12 @@ from typing import Optional
 class PostgresCredentials(Credentials):
     host: str
     user: str
+    role: Optional[str]
     port: Port
     password: str  # on postgres the password is mandatory
     search_path: Optional[str] = None
     keepalives_idle: int = 0  # 0 means to use the default value
+    sslmode: Optional[str] = None
 
     _ALIASES = {
         'dbname': 'database',
@@ -32,7 +35,7 @@ class PostgresCredentials(Credentials):
 
     def _connection_keys(self):
         return ('host', 'port', 'user', 'database', 'schema', 'search_path',
-                'keepalives_idle')
+                'keepalives_idle', 'sslmode')
 
 
 class PostgresConnectionManager(SQLConnectionManager):
@@ -47,8 +50,7 @@ class PostgresConnectionManager(SQLConnectionManager):
             logger.debug('Postgres error: {}'.format(str(e)))
 
             try:
-                # attempt to release the connection
-                self.release()
+                self.rollback_if_open()
             except psycopg2.Error:
                 logger.debug("Failed to release connection!")
                 pass
@@ -58,7 +60,7 @@ class PostgresConnectionManager(SQLConnectionManager):
         except Exception as e:
             logger.debug("Error running SQL: {}", sql)
             logger.debug("Rolling back transaction.")
-            self.release()
+            self.rollback_if_open()
             if isinstance(e, dbt.exceptions.RuntimeException):
                 # during a sql query, an internal to dbt exception was raised.
                 # this sounds a lot like a signal handler and probably has
@@ -88,6 +90,9 @@ class PostgresConnectionManager(SQLConnectionManager):
             kwargs['options'] = '-c search_path={}'.format(
                 search_path.replace(' ', '\\ '))
 
+        if credentials.sslmode:
+            kwargs['sslmode'] = credentials.sslmode
+
         try:
             handle = psycopg2.connect(
                 dbname=credentials.database,
@@ -98,6 +103,9 @@ class PostgresConnectionManager(SQLConnectionManager):
                 connect_timeout=10,
                 **kwargs)
 
+            if credentials.role:
+                handle.cursor().execute('set role {}'.format(credentials.role))
+
             connection.handle = handle
             connection.state = 'open'
         except psycopg2.Error as e:
@@ -114,7 +122,17 @@ class PostgresConnectionManager(SQLConnectionManager):
 
     def cancel(self, connection):
         connection_name = connection.name
-        pid = connection.handle.get_backend_pid()
+        try:
+            pid = connection.handle.get_backend_pid()
+        except psycopg2.InterfaceError as exc:
+            # if the connection is already closed, not much to cancel!
+            if 'already closed' in str(exc):
+                logger.debug(
+                    f'Connection {connection_name} was already closed'
+                )
+                return
+            # probably bad, re-raise it
+            raise
 
         sql = "select pg_terminate_backend({})".format(pid)
 
@@ -130,5 +148,18 @@ class PostgresConnectionManager(SQLConnectionManager):
         return credentials
 
     @classmethod
-    def get_status(cls, cursor):
-        return cursor.statusmessage
+    def get_response(cls, cursor) -> AdapterResponse:
+        message = str(cursor.statusmessage)
+        rows = cursor.rowcount
+        status_message_parts = message.split() if message is not None else []
+        status_messsage_strings = [
+            part
+            for part in status_message_parts
+            if not part.isdigit()
+        ]
+        code = ' '.join(status_messsage_strings)
+        return AdapterResponse(
+            _message=message,
+            code=code,
+            rows_affected=rows
+        )
